/**********************************************************************************
   Copyright(c) Catalyst Semiconductor, Inc., 2005. All rights reserved.

This code is provided as is without warranty or guarantee of any kind as its accuracy, 
completeness, operability, fitness for particular purpose, or any other warranty.

   File        :  MVslave.v
   Description :  Verilog model for CAT93XXXX Microwire serial EEPROMs
		  (viewd as Verilog model for Microwire protocol - slave side)

   Version     :  Rev1.00 
   Release     :  Jan, 2005
**********************************************************************************/

`timescale 1ns/1ns
//
//		cat93cXX chip as slave for Microwire protocol
//
module Chip93cXX(SI,SO,SCK,CSB,ORG,PE);
output SO;
reg SO;
input SI,SCK,CSB,ORG,PE;
`define YES	1
`define NO	0
//
//		states
//
`define idle 				4'b0000
`define receivingInstruction 	4'b0001
`define receivingAdd 		4'b0010
`define receivingData		4'b0111
`define sendingData			4'b1100
//
//		instructions
//
`define READ	3'b110
`define WRITE	3'b101
`define ERASE	3'b111

`define EWEN 	5'b10011
`define EWDS	5'b10000
`define WRAL	5'b10001
`define ERAL	5'b10010
//
//--------------------------------------------------------------------
// 	Read chip type & timing
//
`include "MemType.v"

`ifdef cat93c46
	`include "Timing_46_56_57_66.v"
`endif
`ifdef cat93hc46
	`include "Timing_46_56_57_66.v"
`endif

`ifdef cat93c56
	`include "Timing_46_56_57_66.v"
`endif
`ifdef cat93c57
	`include "Timing_46_56_57_66.v"
`endif
`ifdef cat93c66
	`include "Timing_46_56_57_66.v"
`endif
`ifdef cat93c76
	`include "Timing_76.v"
`endif
`ifdef cat93c86
	`include "Timing_86.v"
`endif

parameter delta = 10;

//-----------------------------------------------------------------------
integer enableWrite,ignoreClock,lastWriteJustDone;
integer bitCounter, seqCounter, i, clockCounter,semClockCounter,semReadSeq,sentDataCounter;
integer lgMem,fullLgCom;
integer man0;
integer tHALF_MIN;
integer outf;
reg [7:0] WREN, WRDI, RDSR, WRSR, READ, WRITE;
reg [4:0] instruction;		
reg [3:0] state,nextState;     	
reg [7:0] mem [0:2047];			
reg [10:0] Saddr,realSaddr;	
reg [7:0] tempReg,dataReg1,dataReg2;
reg [10:0] tempAddrReg; 
reg [10:0] man,man1;
reg [11:0] man2;
reg [50:0] manD;
reg [15:0] manAddr,manAddr1,manRegB; 
reg [8*8:1] chipType;
time savTime,deltaTime;
time negCSBTim;
time posCSBTim;
time negSCKTim;
time posSCKTim;

//####################### TIMING CHECK ###################################
	specify
        //
        //      timing checks for master send activity (IN)
	//
	//	chip select
	$width(negedge CSB, tCSMIN, 0);
	//$setup(posedge CSB, posedge SCK, tCSS);
        //
	// 	clock
	$period(posedge SCK, 1000000/fMAX);	// fMAX in kHz, period in ns
	$width(posedge SCK, tSKHI, 0);
	$width(negedge SCK, tSKLOW, 0);
	//
	// 	data in
	$setup(SI, posedge SCK, tDIS);
	$hold(posedge SCK, SI, tDIH);
        //
	endspecify
//#################################### INITIAL ########################################

initial
	begin

	$display(" ");
	$display("***********************************************************");
	$display("*");

	`ifdef cat93c46		
		chipType = "cat93c46";
		$display("*	Chip:			cat93c46");
		$display("*");
		`define lgMemblk 	128
		`ifdef X8
			$display("*	Memory organization: 	8 bits");
			`define		lgCom	10
			`define 		addrBits 7
			`define 		CaddrBits 7
			lgMem = 128;
		`endif
		`ifdef X16
			$display("*	Memory organization: 	16 bits");
			`define		lgCom	9
			`define 		addrBits  6
			`define 		CaddrBits 6
			lgMem = 64;
		`endif
	`endif

	`ifdef cat93hc46		
		chipType = "cat93hc46";
		$display("*	Chip:			cat93hc46");
		$display("*");
		`define SeqRead	1
		//clock counter (only write instructions 
		// with good sequence length are executed)
		`define ClockCounter	1
		`define lgMemblk 	128
		`ifdef X8
			$display("*	Memory organization: 	8 bits");
			`define		lgCom	10
			`define 		addrBits 7
			`define 		CaddrBits 7
			lgMem = 128;
		`endif
		`ifdef X16
			$display("*	Memory organization: 	16 bits");
			`define		lgCom	9
			`define 		addrBits  6
			`define 		CaddrBits 6
			lgMem = 64;
		`endif
	`endif

	`ifdef cat93c56
		chipType = "cat93c56";
		$display("*             Chip: cat93c56");
		$display("*");
		`define SeqRead	1
		`define lgMemblk 	256
		`ifdef X8
			$display("*	Memory organization: 	8 bits");
			`define		lgCom	12
			`define 		addrBits 8
			`define 		CaddrBits 9
			lgMem = 256;
		`endif
		`ifdef X16
			$display("*	Memory organization: 	16 bits");
			`define		lgCom	11
			`define 		addrBits   7
			`define 		CaddrBits   8
			lgMem = 128;
		`endif
	`endif

	`ifdef cat93c57
		chipType = "cat93c57";
		$display("*             Chip: cat93c57");
		$display("*");
		`define SeqRead	1
		`define lgMemblk 	256
		`ifdef X8
			$display("*	Memory organization: 	8 bits");
			`define		lgCom	11
			`define 		addrBits 8
			`define 		CaddrBits 8
			lgMem = 256;
		`endif
		`ifdef X16
			$display("*	Memory organization: 	16 bits");
			`define		lgCom	10
			`define 		addrBits   7
			`define 		CaddrBits   7
			lgMem = 128;
		`endif
	`endif

	`ifdef cat93c66
		chipType = "cat93c66";
		$display("*             Chip: cat93c66");
		$display("*");
		`define SeqRead	1
		`define lgMemblk 	512
		`ifdef X8
			$display("*	Memory organization: 	8 bits");
			`define		lgCom	12
			`define 		addrBits 9
			`define 		CaddrBits 9
			lgMem = 512;
		`endif
		`ifdef X16
			$display("*	Memory organization: 	16 bits");
			`define		lgCom	11
			`define 		addrBits   8
			`define 		CaddrBits   8
			lgMem = 256;
		`endif
	`endif

	`ifdef cat93c76
		chipType = "cat93c76";
		$display("*             Chip: cat93c76");
		$display("*");
		`define SeqRead	1
		//clock counter (only write instructions 
		// with good sequence length are executed)
		`define ClockCounter	1
		`define lgMemblk 	1024
		`ifdef X8
			$display("*	Memory organization: \t\t8 bits");
			`define		lgCom	14
			`define 		addrBits 10
			`define 		CaddrBits 11
			lgMem = 1024;
		`endif
		`ifdef X16
			$display("*	Memory organization: \t\t16 bits");
			`define		lgCom	13
			`define 		addrBits 9
			`define 		CaddrBits 10
			lgMem = 512;
		`endif
	`endif

	`ifdef cat93c86
		chipType = "cat93c86";
		$display("*             Chip: cat93c86");
		$display("*");
		`define SeqRead	1
		`define lgMemblk 	2048
		`ifdef X8
			$display("*	Memory organization: \t\t8 bits");
			`define		lgCom	14
			`define 		addrBits 11
			`define 		CaddrBits 11
			lgMem = 2048;
		`endif
		`ifdef X16
			$display("*	Memory organization: \t\t16 bits");
			`define		lgCom	13
			`define 		addrBits 10
			`define 		CaddrBits 10
			lgMem = 1024;
		`endif
	`endif

	manAddr = lgMem;
	manAddr1 = manAddr - 1;
	tHALF_MIN = (1000000)/(2*fMAX); 
	`ifdef X8
		$display("*	Memory size:\t\t\t%d bytes (0 h - %h h)",manAddr,manAddr1);
	`endif
	`ifdef X16
		$display("*	Memory size:\t\t\t%d words (0 h - %h h)",manAddr,manAddr1);
	`endif	
	`ifdef ClockCounter
		semClockCounter = 1;
	`else
		semClockCounter = 0;
	`endif
	manRegB = `lgCom;
	$display("*	Instruction length:\t\t%d bits",manRegB);
	$display("*");

	`ifdef v1_8__6_0
		$display ("*\tTiming for Vcc = 1.8 V - 6.0 V");
	`endif
	`ifdef v2_5__6_0
		$display ("*\tTiming for Vcc = 2.5 V - 6.0 V");
	`endif
	`ifdef v4_5__5_5
		$display ("*\tTiming for Vcc = 4.5 V - 5.5 V");
	`endif
	$display("*");

	manD = fMAX;
	$display("* fMAX = \t%g KHz\t\t(max. frequency )",manD);
	manD = tSKHI;
	$display("* tSKHI =\t%d ns\t\t(min. SK High Time)",manD);
	manD = tSKLOW;
	$display("* tSKLOW=\t%d ns\t\t(min. SK Low Time)",manD);
	manD = tCSS;
	$display("* tCSS = \t%d ns\t\t(min. CS Setup Time)",manD);
	manD = tCSH;
	$display("* tCSH = \t%d ns\t\t(min. CS HOLD Time)",manD);
	manD = tCSMIN;
	$display("* tCSMIN=\t%d ns\t\t(min. CS Low Time)",manD);
	manD = tDIS;
	$display("* tDIS = \t%d ns\t\t(min. Data In Setup Time)",manD);
	manD = tDIH;
	$display("* tDIH = \t%d ns\t\t(min. Data In Hold Time)",manD);
	manD = tPD1;
	$display("* tPD1 = \t%d ns\t\t(max. Output Delay to 1)",manD);
	manD = tPD0;
	$display("* tPD2 = \t%d ns\t\t(max. Output Delay to 0)",manD);
	manD = tHZ;
	$display("* tHZ = \t%d ns\t\t(max. Output Delay to High-Z)",manD);
	manD = tSV;
	$display("* tSV = \t%d ns\t\t(max. Output Delay to Status Valid)",manD);
	manD = tEW;
	$display("* tEW = \t%d ns\t\t(max. Program/Erase Pulse Width)",manD);
	$display("*");

	semReadSeq = 1;

	`ifdef SeqRead
			$display("* Sequential read allowed");
	`else
			$display("* Sequential read not allowed");
			semReadSeq = 0;
	`endif
	`ifdef ClockCounter
			$display("* Clock counter enabled (check on write instructions sequence length)");
	`else
			$display("* Clock counter disabled (no check on write instructions sequence length)");
	`endif

	`ifdef cat93c86
		`ifdef PEset
				$display("*   %s chip: PE pin = 1 (write allowed)",chipType);
		`else
				$display("*   %s chip: PE pin = 0 (write not allowed)",chipType);
		`endif
	`endif

	//
	// memory initialization
	//
	`ifdef initMemFile
		//
		// memory initialization with data from file
		//
		$readmemh(`initMemFile,mem);
		man2 = `lgMemblk;
		$display("*");
		$display("* Memory (%d bytes) initialization from \"%s\"",man2,`initMemFile);
	`else
		//
		// memory initialization with 00
		//
		for(i=0; i<`lgMemblk; i=i+1)
			begin
			mem[i] = 8'h00;
			end
		man2 = `lgMemblk;
		$display("*");
		$display("* Memory (%d bytes) initialization with 8'h00...",man2);
	`endif
	`ifdef dumpMemFile
		//
		// memory dump to file
		//
		$display("* Memory (%d bytes) will be dumped, after each write, to \"%s\"\n",man2,`dumpMemFile);
	`endif

	//
	state = `idle;
	nextState = `idle;
	Saddr = 16'h0000;
	instruction = 8'h00;
	bitCounter = 0;
	SO = 1'bz;
	enableWrite = `NO;
	ignoreClock = `NO;
	lastWriteJustDone = `NO;
end

//###################### ALWAYS ###################################

always @(negedge CSB) 
	negCSBTim = $time;

always @(posedge CSB) 
	posCSBTim = $time;

always @(negedge SCK) 
	negSCKTim = $time;

always @(posedge SCK) 
	posSCKTim = $time;

always @(negedge CSB) 
   	begin
   	SO = 1'bz;
   	bitCounter = 0;
   	end

always @(posedge CSB)
	begin
	if(instruction[4:0]==`WRAL || instruction[2:0]==`WRITE)
		begin
		if(clockCounter != 0)
			if(fullLgCom == clockCounter || (fullLgCom != clockCounter && semClockCounter == 0))
				enableWrite = `YES;
			else
				begin
				enableWrite = `NO;
				man = fullLgCom;
				man1 = clockCounter;
				$display("*** %s chip: \tCLOCK COUNTER ERROR\n    (received instruction sequence has %d bits instead of %d bits) ",chipType,man1,man,$time);
				end
		end
	if(instruction[4:0]==`ERAL || instruction[2:0]==`ERASE)
		begin
		if(clockCounter != 0)
			if(`lgCom == clockCounter || (`lgCom != clockCounter && semClockCounter == 0))
				begin
				enableWrite = `YES;
				end
			else
				begin
				enableWrite = `NO;
				man = `lgCom;
				man1 = clockCounter;
				$display("*** %s chip: \tClock counter error (received instruction sequence has %d bits instead %d bits)",chipType,clockCounter,man);
				end
		end



	if(lastWriteJustDone == `NO && enableWrite == `YES && (instruction[4:0]==`ERAL || instruction[4:0]==`WRAL || instruction[2:0]==`WRITE || instruction[2:0]==`ERASE)) 
		begin
			if(chipType != "cat93c86" || (chipType == "cat93c86" && PE == 1))
				begin
				ignoreClock = `YES;
				#tSV SO = 0;				// Busy
				$display("    %s chip: \tBegin internal writing ...",chipType,$time);
				//-------------------------------------------------------------------
				case(instruction[2:0])
					`WRITE:
						begin
						`ifdef X8
							mem[realSaddr] = dataReg1[7:0];
							realSaddr[`addrBits-1:0] = realSaddr[`addrBits-1:0] + 1;
						`endif
						`ifdef X16
							mem[realSaddr] = dataReg1[7:0];
							realSaddr[`addrBits:0] = realSaddr[`addrBits:0] + 1;
							mem[realSaddr] = dataReg2[7:0];
							realSaddr[`addrBits:0] = realSaddr[`addrBits:0] + 1;
						`endif
						end
					`ERASE:
						begin
						mem[realSaddr] = 8'hff;
						`ifdef X16
							mem[realSaddr+1] = 8'hff;
						`endif
						end
				endcase
				case(instruction[4:0])
					`WRAL:
						begin
						`ifdef X8
						for(i=0; i<`lgMemblk; i=i+1)
							mem[i] = dataReg1[7:0];
						`endif
						`ifdef X16
						for(i=0; i<`lgMemblk; i=i+2)
							mem[i] = dataReg1[7:0];
						for(i=1; i<`lgMemblk; i=i+2)
							mem[i] = dataReg2[7:0];
						`endif
						end
					`ERAL:
						begin
						for(i=0; i<`lgMemblk; i=i+1)
							mem[i] = 8'hff;
						$display("    %s chip: \tAll memory bits will be set to 1",chipType);
						end
				endcase
				#(tEW/1000-tCSMIN-tSV) SO = 1; 	// Ready
				//------------------------------------------------------------------
				$display("    %s chip: \t... end internal writing",chipType,$time);
				//---
				`ifdef dumpMemFile
				//
				// memory dump to file
				//
					outf = $fopen(`dumpMemFile);
					for(i=0; i<`lgMemblk; i=i+1)
						begin
						$fdisplay(outf,"@%h",i);
						$fdisplay(outf,"\t%h",mem[i]);
						end
					$fclose(outf);
				$display("* Memory content dumped to \"%s\" file\n",`dumpMemFile);
				`endif
				//----
				ignoreClock = `NO;
				state = `idle;
				nextState = `idle;
				bitCounter = 0;
				end
		end
	#0 if((posCSBTim - negCSBTim) >= tCSMIN)
		begin
		if(SCK == 1'b0)
			begin
			nextState = `receivingInstruction;
			end
		else
			begin
			#tHALF_MIN nextState = `receivingInstruction;
			end
		lastWriteJustDone = `YES;
		clockCounter = 0;
		end
	else
		begin
		$display("***** SLAVE:  WARNING ");
		$display("              START ignored (tCSMIN - Min.CS Low Time - not respected: %d < tCS) ",posCSBTim - negCSBTim,$time);
		end
	end

always @(posedge SCK)
	begin
   if(CSB ==1 && ignoreClock == `NO && nextState != `idle)
   begin

	clockCounter = clockCounter + 1;

	state = nextState;

	case(state)

		`receivingInstruction:
			begin
			bitCounter = bitCounter + 1;
			case(bitCounter)
			1:
				begin
				instruction = 0;
				tempReg = 0;
				tempReg[7:0] = 8'b00000000;
				if((posSCKTim-posCSBTim) >= tCSS)
					begin
					#0 tempReg[0] = SI;
					tempReg = tempReg << 1;
					#delta if(SI != 1)
						begin
						$display("*** %s chip: \tReceived start bit is not 1 !",chipType,$time); 
						bitCounter = bitCounter -1 ;
						clockCounter = clockCounter - 1;
						end
					end
				else
					begin
					$display("    %s chip: \tSTART ignored (tCSS - CS Setup Time - not respected: %d < tCSS) ", chipType,posSCKTim-posCSBTim,$time);
					bitCounter = 0;
					nextState = `idle;
					end
				end
			3:
				begin
				tempReg[0] = SI;
				instruction[2:0] = tempReg[2:0];
				case(instruction[2:0])
					`READ:
						begin
						instruction[2:0] = tempReg[2:0];
						$display("    %s chip: \tReceived instruction code: 	 %b ", chipType,instruction[1:0]);
						tempAddrReg = 0;
						nextState = `receivingAdd;
						#(tHALF_MIN*3/2) bitCounter = 0;
						end
					`WRITE:
						begin
						instruction[2:0] = tempReg[2:0];
						$display("    %s chip: \tReceived instruction code: 	 %b ", chipType,instruction[1:0]);
						tempAddrReg = 0;
						nextState = `receivingAdd;
						`ifdef X8
							seqCounter = 1;
						`endif
						`ifdef X16
							seqCounter = 2;
						`endif
						lastWriteJustDone = `NO;
						#(tHALF_MIN*3/2) bitCounter = 0;
						end
					`ERASE:
						begin
						instruction[2:0] = tempReg[2:0];
						$display("    %s chip: \tReceived instruction code: 	 %b ", chipType,instruction[1:0]);
						tempAddrReg = 0;
						nextState = `receivingAdd;
						lastWriteJustDone = `NO;
						#(tHALF_MIN*3/2) bitCounter = 0;
						end

					default:
						begin
						tempReg[0] = SI;
						tempReg = tempReg << 1;
						nextState = `receivingInstruction;
						end
				endcase
				end
			5:
				begin
				tempReg[0] = SI;
				instruction[4:0] = tempReg[4:0];
				$display("    %s chip: \tReceived instruction code: 	 %b ", chipType,instruction[3:0]);
				case(instruction[4:0])
					`EWEN:
						begin
						end
					`EWDS:
						begin
						end
					`ERAL:
						begin
						lastWriteJustDone = `NO;
						end
					`WRAL:
						begin
						`ifdef X8
							seqCounter = 1;
						`endif
						`ifdef X16
							seqCounter = 2;
						`endif
						lastWriteJustDone = `NO;
						end

					default:
						begin
						tempReg[0] = SI;
						tempReg = tempReg << 1;
						$display("*** %s chip: \tWrong received instruction code (%b) !",chipType,instruction);
						end
					endcase
				tempAddrReg = 0;
				nextState = `receivingAdd;
				#(tHALF_MIN*3/2) bitCounter = 0;
				end

			default:
				begin
				tempReg[0] = SI;
				tempReg = tempReg << 1;
				end
			endcase
			end

		`receivingAdd:
			begin
			bitCounter = bitCounter + 1;
			case(bitCounter)
			`CaddrBits-2:
				begin
				if(instruction[4:0]==`EWEN || instruction[4:0]==`EWDS || instruction[4:0]==`ERAL)
					begin
					if(`lgCom == clockCounter || (`lgCom != clockCounter && semClockCounter == 0))
						begin
						case(instruction[4:0])
						`EWEN:
							begin
							enableWrite = `YES;
							$display("    %s chip: \tWrite enabled",chipType);
							end
						`EWDS:
							begin
							enableWrite = `NO;
							$display("    %s chip: \tWrite disabled",chipType);
							end
						`ERAL:
							begin
							end
						endcase
						end
					else
						begin
						case(instruction[4:0])
						`EWEN:
							begin
							enableWrite = `YES;
							$display("    %s chip: \tWrite enabled",chipType);
							end
						`EWDS:
							begin
							enableWrite = `NO;
							$display("    %s chip: \tWrite disabled",chipType);
							end
						`ERAL:
							begin
							end
						endcase
						end
					///nextState = `receivingInstruction;
					nextState = `idle;
					#(tHALF_MIN*3/2) bitCounter = 0;
					end
				else
					if(instruction[4:0]==`WRAL)
						begin
						nextState = `receivingData;
						#(tHALF_MIN*3/2) bitCounter = 0;
						end
					else
						begin
						tempAddrReg[0] = SI;
						tempAddrReg = tempAddrReg << 1;
						end
				end
			`CaddrBits:
				begin
				tempAddrReg[0] = SI;
				Saddr = 0;
				Saddr[`addrBits-1:0] = tempAddrReg[`addrBits-1:0];
				$display("    %s chip: \tReceived addr[ %d:0]:             %b (%hH)",chipType,`addrBits-1,Saddr[`addrBits-1:0],Saddr[`addrBits-1:0]);		

				`ifdef X8
					realSaddr = Saddr;
				`endif
				`ifdef X16
					realSaddr = 2*Saddr;
				`endif

				case(instruction[2:0])
					`READ:
						begin
						//SO = 0;
						SO = SO;
						sentDataCounter = 0;
						nextState = `sendingData;
						#(tHALF_MIN*3/2) bitCounter = 0;
						end
					`ERASE:
						begin
						#(tHALF_MIN*3/2) bitCounter = 0;
						end
					`WRITE:
						begin
						nextState = `receivingData;
						#(tHALF_MIN*3/2) bitCounter = 0;
						end
				endcase

				end
			default:
				begin
				tempAddrReg[0] = SI;
				tempAddrReg = tempAddrReg << 1;
				end
			endcase
			end

		`receivingData:
			begin
			bitCounter = bitCounter + 1;
			if(bitCounter == 9)
				bitCounter = 1;
			case(bitCounter)
			1:
				begin
				tempReg[7:0] = 8'b00000000;
				#0 tempReg[0] = SI;
				tempReg = tempReg << 1;
				end
			8:
				begin
				seqCounter = seqCounter - 1;
				#0
				tempReg[0] = SI;
				`ifdef X8
					dataReg1[7:0] = tempReg[7:0];
					$display("    %s chip: \tReceived data: %h",chipType,dataReg1[7:0]);
					fullLgCom = `lgCom + 8;
				`endif
				`ifdef X16
					fullLgCom = `lgCom + 16;
					if(seqCounter == 1)
						begin
						dataReg1[7:0] = tempReg[7:0];
						$display("    %s chip: \tReceived data: %h",chipType,dataReg1[7:0]);
						fullLgCom = `lgCom + 8;
						end
					if(seqCounter == 0)
						begin
						dataReg2[7:0] = tempReg[7:0];
						$display("    %s chip: \tReceived data: %h",chipType,dataReg2[7:0]);
						end
				`endif

				if(instruction[2:0] == `WRITE)
					begin
					#(2*tHALF_MIN) 
					state = `receivingData;
					nextState = `receivingData;
					end	

				if(instruction[4:0] ==`WRAL)
					begin
					#(2*tHALF_MIN) 
					state = `receivingData;
					nextState = `receivingData;
					end
				end

			default:
				begin
				tempReg[0] = SI;
				tempReg = tempReg << 1;
				end
			endcase
			end
	endcase
    end
	end

always @(negedge SCK)
	begin
	savTime = $time;
	if(CSB == 1 && ignoreClock == `NO && nextState != `idle)
    begin
	state = nextState;
	case(state)
		`sendingData:
			begin
			#(tHALF_MIN+delta) if(state == `sendingData)
				begin
				deltaTime = $time - savTime;
				if($time-negCSBTim > deltaTime)
					//SO = 1'b0;
					SO = SO;
				while(SCK==0)
					#1 SO=SO;
				//___ now, posedge of SCK
				deltaTime = $time - savTime;
				if($time-negCSBTim > deltaTime)
					begin
					bitCounter = bitCounter + 1;
					case(bitCounter)
						1:
							begin
							tempReg[7:0] = mem[realSaddr];
							#(tPD1-2*delta) SO = tempReg[7];
							#0 tempReg = tempReg << 1;
							end
						2:
							begin
							#(tPD1-2*delta) SO = tempReg[7];
							#0 tempReg = tempReg << 1;
							end
						8:
							begin
							#(tPD1-2*delta) SO = tempReg[7];
							#0 tempReg = tempReg << 1;
							sentDataCounter = sentDataCounter + 1;
							//#(tHALF_MIN-tPD1) bitCounter = 0;
							#(tSKHI-tPD1) bitCounter = 0;
							`ifdef X8
								realSaddr[`addrBits-1:0] = realSaddr[`addrBits-1:0] + 1;	//(inc.adr.in mem.)
							`endif
							`ifdef X16
								realSaddr[`addrBits:0] = realSaddr[`addrBits:0] + 1;	//(inc.adr.in mem.)
							`endif
							if(semReadSeq == 0)
								begin
								`ifdef X8
								nextState = `idle;
								`endif
								`ifdef X16
								if(sentDataCounter == 1)
									nextState = `sendingData;
								else
									nextState = `idle;
								`endif
								end
							else
								nextState = `sendingData;
							end
						default:
							begin
							#(tPD1-2*delta) SO = tempReg[7];
							#0 tempReg = tempReg << 1;
							end
					endcase
				end

				end
			end
		endcase
    end
end

endmodule
